local addonName, ns = ...
local _G = _G
local MODULE_NAME = 'PlayerXP'
local addon = ns.addon
local module = addon:createModule(MODULE_NAME, addon.xpBarTemplate, 'AceEvent-3.0')
local setfenv = _G.setfenv
setfenv(1, module)

local min = _G.min
local assert = _G.assert
local string, tostring = _G.string, _G.tostring

local UnitXP = _G.UnitXP
local UnitXPMax = _G.UnitXPMax
local UnitLevel = _G.UnitLevel
local GetMaxPlayerLevel = _G.GetMaxPlayerLevel
local GetXPExhaustion = _G.GetXPExhaustion

local XP_PER_LEVEL = ns.XP_PER_LEVEL

-- start xp
function module:getStartXPMin()
    return 0
end

-- target xp
function module:getTargetXPMax()
    local level = UnitLevel('player')
    local maxPlayerLevel = GetMaxPlayerLevel()
    if level < maxPlayerLevel then
        return self:calcXpForLevel(maxPlayerLevel)
    else
        return self:getCurrentXP() + 1
    end
end

-- target level
function module:getTargetLevelMax()
    return GetMaxPlayerLevel()
end

-- end of options

module.oldEnable = module.OnEnable
function module:OnEnable()
    self:oldEnable()
    self:RegisterEvent('UPDATE_EXHAUSTION', 'Invalidate')
    self:RegisterEvent('PLAYER_XP_UPDATE', 'Invalidate')
end

function module:Redraw()
    --addon:Debug("Player XP Redraw")

    local xpOptions = self:getOptions()
    local barCache = self.cache

    local maxPlayerLevel = GetMaxPlayerLevel()
    if maxPlayerLevel == 0 then
        self:Invalidate()
        return
    end

    local playerLevel = UnitLevel('player')
    local currentGlobalXP = self:getCurrentXP()
    -- update max settings if necessary
    if xpOptions.targetXP and currentGlobalXP > xpOptions.targetXP then
        xpOptions.targetXP = currentGlobalXP
    end
    local startGlobalXP = xpOptions.startXP
    local targetGlobalXP = xpOptions.targetXP or self:calcXpForLevel(maxPlayerLevel)
    local restedGlobalXP = GetXPExhaustion() or 0
    --if not restedGlobalXP then
    --    self:Invalidate()
    --    return
    --end
    local currentGlobalXPRatio = (currentGlobalXP-startGlobalXP)/(targetGlobalXP-startGlobalXP)
    local restedGlobalXPRatio =  (restedGlobalXP               )/(targetGlobalXP-startGlobalXP)
    -- create textures
    assert(barCache and barCache.frame)
    local frame = barCache.frame
    local right = frame:GetRight()
    local left = frame:GetLeft()
    if not (right and left) then
        self:Invalidate()
        return
    end
    local width = frame:GetRight() - frame:GetLeft()
    --addon:Debug(width)
    -- texture for curent xp
    self:DrawPrimaryBar(currentGlobalXPRatio)

    -- texture for rested xp
    local restedRatio = min(restedGlobalXPRatio,1-currentGlobalXPRatio)
    self:DrawSecondaryBar(restedRatio)

    -- textures and numbers for levels
     self:drawNumbers()

    --update options table
    self:updateOptionsTable()
end

function module:getCurrentXP()
    local xpInCurrentLevel = UnitXP('player')
    local level = UnitLevel('player')
    return xpInCurrentLevel + self:calcXpForLevel(level)
end

function module:calcXpForLevel(level)
    local sum = 0
    assert(level and XP_PER_LEVEL[level],"no data for level=" .. tostring(level))
    for i=1,level-1 do
        sum = sum + XP_PER_LEVEL[i]
    end
    return sum
end

function module:calcLevelForXP(xp)
    local level = 1
    local maxPlayerLevel = GetMaxPlayerLevel()
    while level < maxPlayerLevel and xp > self:calcXpForLevel(level+1) do
        level = level + 1
    end
    local startXP = self:calcXpForLevel(level)
    assert(startXP <= xp,"level=" .. level .. " ,xp=" .. xp .. " ,startXP=" .. startXP)
    if startXP == xp or level == maxPlayerLevel then
        return level
    end
    local nextXP = self:calcXpForLevel(level+1)
    local fraction = (xp-startXP)/(nextXP-startXP)
    assert(0.0 <= fraction and fraction <= 1.0, "fraction=" .. fraction .. ", level=" .. level .. " ,xp=" .. xp .. " ,startXP=" .. startXP)
    assert(1 <= level and level <= maxPlayerLevel,"level=" .. level)
    return level + fraction
end

function module:fillTooltip(tooltip)
    local currentGlobalXP = self:getCurrentXP()
    local targetGlobalXP = self:getOptions().targetXP or self:getTargetXPMax()
    local startGlobalXP = self:getOptions().startXP
    local deltaXP = targetGlobalXP-startGlobalXP
    local restedGlobalXP = GetXPExhaustion() or 0
    self:addNameToTooltip(tooltip)
    self:fillTooltipWithSecond(tooltip, currentGlobalXP-startGlobalXP, deltaXP, restedGlobalXP)
end

